package cc.bnblogs.usercenterbackend.controller;

import cc.bnblogs.usercenterbackend.common.BaseResponse;
import cc.bnblogs.usercenterbackend.common.ErrorCode;
import cc.bnblogs.usercenterbackend.common.ResultUtils;
import cc.bnblogs.usercenterbackend.exception.BusinessException;
import cc.bnblogs.usercenterbackend.model.User;
import cc.bnblogs.usercenterbackend.model.request.UserLoginRequest;
import cc.bnblogs.usercenterbackend.model.request.UserRegisterRequest;
import cc.bnblogs.usercenterbackend.service.UserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.stream.Collectors;

import static cc.bnblogs.usercenterbackend.constant.UserConstant.ADMIN_ROLE;
import static cc.bnblogs.usercenterbackend.constant.UserConstant.LOGIN_USER;

/**
 * @description: 用户接口
 * @author: zfp@bnblogs.cc
 * @date: 2023/3/13 20:42
 */
@RestController
@RequestMapping("/user")
public class UserController {
    @Resource
    private UserService userService;

    /**
     * 用户注册
     * @param request 用户注册请求体参数
     * @return 注册成功的用户id
     */
    @PostMapping("/register")
    public BaseResponse<Long> register(@RequestBody UserRegisterRequest request) {
        if (request == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"客户端请求为null");
        }
        String userAccount = request.getUserAccount();
        String userPassword = request.getUserPassword();
        String checkPassword = request.getCheckPassword();
        String planetCode = request.getPlanetCode();


        /*
          controller层可以对请求参数做一定的校验
         */
        if (StringUtils.isAnyBlank(userAccount, userPassword, checkPassword,planetCode)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"请求的字段中有参数为空");
        }
        Long result = userService.userRegister(userAccount,userPassword,checkPassword,planetCode);
        return ResultUtils.success(result);
    }

    /**
     * 用户登录
     * HttpServletRequest对象代表客户端的请求，当客户端通过HTTP协议访问服务器时，
     * HTTP请求头中的所有信息都封装在这个对象中，通过这个对象提供的方法，可以获得客户端请求的所有信息。
     * @param request 用户登录请求体参数
     * @param httpServletRequest 客户端请求
     * @return 脱敏后的用户信息
     */
    @PostMapping("/login")
    public BaseResponse<User> login(@RequestBody UserLoginRequest request, HttpServletRequest httpServletRequest) {
        if (request == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"客户端请求为null");
        }
        String userAccount = request.getUserAccount();
        String userPassword = request.getUserPassword();

        /*
          controller层可以对请求参数做一定的校验
         */
        if (StringUtils.isAnyBlank(userAccount, userPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名或密码为null或为空");
        }
        User user = userService.userLogin(userAccount, userPassword, httpServletRequest);
        return ResultUtils.success(user);
    }

    /**
     * 用户注销
     * @param request 客户端发起的请求
     * @return 注销结果，注销成功返回1
     */
    @PostMapping("/logout")
    public BaseResponse<Integer> login(HttpServletRequest request) {
        if (request == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"客户端请求为null");
        }
        int res = userService.userLogout(request);
        return ResultUtils.success(res);
    }

    /**
     * 根据用户名进行模糊查询
     * 该接口调用前需要鉴权，需要判断用户是否可以调用该接口
     * @param username 用户名
     * @param request 客户端请求
     * @return 用户列表
     */
    @GetMapping("/search")
    public BaseResponse<List<User>> getUsersByName(String username,
                                HttpServletRequest request) {
        // 仅管理员可使用本接口
        if (!isAdmin(request)) {
            // todo: 这里应该返回空的列表
            throw new BusinessException(ErrorCode.NOT_ADMIN,"本接口需要管理员权限");
        }

        QueryWrapper<User> qw = new QueryWrapper<>();
        // 用户名不为空
        if (StringUtils.isNotBlank(username)) {
            qw.like("username", username);
        }
        List<User> userList = userService.list(qw);
        // 返回脱敏后的用户列表
        List<User> lists = userList.stream().map(user -> userService.getSafetyUser(user)).collect(Collectors.toList());
        return ResultUtils.success(lists);
    }

    /**
     * 根据用户id逻辑删除某一个用户
     * @param id 用户id
     * @return 删除结果
     */
    @DeleteMapping("/delete")
    public BaseResponse<Boolean> deleteUserById(@RequestBody Long id,HttpServletRequest request) {
        if (!isAdmin(request)) {
            throw new BusinessException(ErrorCode.NOT_ADMIN,"需要管理员权限");
        }

        if (id <= 0) {
           throw  new BusinessException(ErrorCode.PARAMS_ERROR,"删除的用户id不合法");
        }

        /*
            注意: 这里是逻辑删除不是物理删除
         */
        boolean result = userService.removeById(id);
        return ResultUtils.success(result);
    }

    /**
     * 返回当前用户的信息
     * @param request
     * @return
     */
    @GetMapping("/current")
    public BaseResponse<User> getCurrentUser(HttpServletRequest request) {
        Object userObject = request.getSession().getAttribute(LOGIN_USER);
        if (userObject == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN,"用户未登录");
        }
        // 从session中取出用户id，再根据用户id去查询数据库中的用户数据(这样相当于更新了缓存)
        User user = (User) userObject;
        Long id = user.getId();
        // todo: 校验用户是否合法，比如是否被封号了
        User originUser = userService.getById(id);
        // 返回脱敏后的用户数据
        User safetyUser = userService.getSafetyUser(originUser);
        return ResultUtils.success(safetyUser);
    }

    /**
     * 判断是否为管理员
     * @param request 客户端请求
     * @return
     */
    public boolean isAdmin(HttpServletRequest request) {
        // 从session中获取用户信息
        Object userObject = request.getSession().getAttribute(LOGIN_USER);
        User user = (User) userObject;

        // 仅管理员可查询
        return user != null && user.getUserRole().equals(ADMIN_ROLE);
    }
}
