package cc.bnblogs.usercenterbackend.service.impl;

import cc.bnblogs.usercenterbackend.common.ErrorCode;
import cc.bnblogs.usercenterbackend.exception.BusinessException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cc.bnblogs.usercenterbackend.model.User;
import cc.bnblogs.usercenterbackend.service.UserService;
import cc.bnblogs.usercenterbackend.mapper.UserMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static cc.bnblogs.usercenterbackend.constant.UserConstant.LOGIN_USER;

/**
* @author zfp@bnblogs.cc
* @description 针对表【user(用户)】的数据库操作Service实现
* @createDate 2023-03-13 17:18:58
*/
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
    implements UserService{

    /**
     * 有效的账户名的正则表达式
     */
    private static final String ACCOUNT_VALID= "^[a-zA-Z][a-zA-Z0-9_]{3,}";

    /**
     * 加密使用到的盐值
     */
    private static final String SALT = "salt";


    @Resource
    private UserMapper userMapper;

    @Override
    public long userRegister(String userAccount, String userPassword, String checkPassword,String planetCode) {
        // 1.校验字段
        // 字段值不能为空
        if (StringUtils.isAnyBlank(userAccount,userPassword,checkPassword,planetCode)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"字段值不能为空");
        }
        // 账户名不少于4位
        if (userAccount.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名不能小于4位");
        }
        // 账户名只能使用字母数字下划线,且必须字母开头
        Matcher matcher = Pattern.compile(ACCOUNT_VALID).matcher(userAccount);

        if (!matcher.find()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名要以英文字母开头");
        }

        // 星球编号不能过长
        if (planetCode.length() > 8) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"星球编号不超过8位");
        }
        // 密码不少于8位
        if (userPassword.length() < 8 || checkPassword.length() < 8) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"密码不能少于8位");
        }

        // 两次输入的密码不一致
        if (!userPassword.equals(checkPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"两次输入的密码不一致");
        }

        // 账户不能重复, 涉及数据库操作的检验放在靠后的地方
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw.eq("userAccount",userAccount);
        long count = userMapper.selectCount(qw);
        if (count > 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名不能重复");
        }

        // 星球编号不能重复
        qw = new QueryWrapper<>();
        qw.eq("planetCode",planetCode);
        count = userMapper.selectCount(qw);
        if (count > 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"星球编号已存在");
        }

        //2.对原始密码进行加密
        String encryptPassword = DigestUtils.md5DigestAsHex(( SALT+ userPassword).getBytes());


        //3.存入数据库中
        User user = new User();
        user.setUserAccount(userAccount);
        // 测试阶段: 设置用户名和账户名称相同
        user.setUsername(userAccount);
        user.setUserPassword(encryptPassword);
        user.setPlanetCode(planetCode);
        int saveResult = userMapper.insert(user);
        // 插入数据失败
        if (saveResult == 0) {
            return -1;
        }
        return user.getId();
    }

    @Override
    public User userLogin(String userAccount, String userPassword, HttpServletRequest request) {
        // 1.校验字段
        // 字段值不能为空
        if (StringUtils.isAnyBlank(userAccount,userPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"字段值不能为空");
        }
        // 账户名不少于4位
        if (userAccount.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名不能小于4位");
        }
        // 账户名只能使用字母数字下划线,且必须字母开头
        Matcher matcher = Pattern.compile(ACCOUNT_VALID).matcher(userAccount);

        if (!matcher.find()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"账户名要以英文字母开头");
        }

        // 密码不少于8位
        if (userPassword.length() < 8) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"密码不能少于8位");
        }

        // 2.密码加密
        String encryptPassword = DigestUtils.md5DigestAsHex(( SALT+ userPassword).getBytes());


        // 根据用户名和密码查看是否存在该用户
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw.eq("userAccount",userAccount);
        qw.eq("userPassword",encryptPassword);
        User user = userMapper.selectOne(qw);

        // 用户名或密码不匹配
        if (user == null) {
            throw new BusinessException(ErrorCode.NULL_ERROR,"用户名或者密码不匹配");
        }

        // 3.用户数据脱敏
        User safetyUser = getSafetyUser(user);


        // 4.记录用户的登录状态,保留脱敏后的信息
        request.getSession().setAttribute(LOGIN_USER,safetyUser);
        log.info("登录成功!");

        return safetyUser;
    }

    /**
     * 用户数据脱敏
     * @param originUser 未进行脱敏的用户
     * @return 脱敏后的用户
     */
    @Override
    public User getSafetyUser(User originUser) {
        if (originUser == null) {
            return null;
        }
        User safetyUser = new User();
        safetyUser.setId(originUser.getId());
        safetyUser.setUsername(originUser.getUsername());
        safetyUser.setUserAccount(originUser.getUserAccount());
        safetyUser.setAvatarUrl(originUser.getAvatarUrl());
        safetyUser.setGender(originUser.getGender());
        safetyUser.setPhone(originUser.getPhone());
        safetyUser.setEmail(originUser.getEmail());
        safetyUser.setUserStatus(originUser.getUserStatus());
        safetyUser.setCreateTime(originUser.getCreateTime());
        safetyUser.setUserRole(originUser.getUserRole());
        safetyUser.setPlanetCode(originUser.getPlanetCode());
        return safetyUser;
    }

    @Override
    public int userLogout(HttpServletRequest request) {
        // 移除session中的登录态
        request.getSession().removeAttribute(LOGIN_USER);
        log.info("注销成功!");
        return 1;
    }

}




